@@ -4,18 +4,12 @@ use warnings;
 use strict;
 use Catalyst::Exception;
 
+our $VERSION = '0.26';
+
 =head1 NAME
 
 CatalystX::CRUD - CRUD framework for Catalyst applications
 
-=head1 VERSION
-
-Version 0.01
-
-=cut
-
-our $VERSION = '0.25';
-
 =head1 DESCRIPTION
 
 This document is an overview of the CatalystX::CRUD framework and API.
@@ -61,7 +55,7 @@ processing instead of returning. See fetch() for an example.
 sub throw_error {
     my $self = shift;
     my $msg = shift || 'unknown error';
-    Catalyst::Exception->throw($msg);
+    Carp::confess($msg);
 }
 
 =head1 AUTHOR
@@ -106,6 +100,8 @@ L<http://search.cpan.org/dist/CatalystX-CRUD>
 
 =head1 ACKNOWLEDGEMENTS
 
+Thanks to Zbigniew Lukasiak and Matt Trout for feedback and API ideas.
+
 =head1 COPYRIGHT & LICENSE
 
 Copyright 2007 Peter Karman, all rights reserved.
