@@ -1,28 +1,24 @@
 package MyApp::Controller::File;
 use strict;
-use base qw( CatalystX::CRUD::Controller );
+use base qw( CatalystX::CRUD::Test::Controller );
 use Carp;
 use Data::Dump qw( dump );
 use File::Temp;
+use MyApp::Form;
+
+__PACKAGE__->config(
+    primary_key => 'absolute',
+    form_class  => 'MyApp::Form',
+    form_fields => [qw( file content )],
+    model_name  => 'File',
+    primary_key => 'file',
+    init_form   => 'init_with_file',
+    init_object => 'file_from_form',
+);
 
 # test the view_on_single_result method
 # search for a file where we know there is only one
 # and then check for a redirect response code
-# NOTE we have to fake up the primary_key method
-# to just return the file path (the unique id)
-# and the form class to just use a dummy
-
-{
-
-    package NoForm;
-    sub new { return bless( {}, shift(@_) ); }
-}
-
-__PACKAGE__->config(
-    primary_key => 'absolute',
-    form_class  => 'NoForm',
-    model_name  => 'File',
-);
 
 sub do_search {
 
@@ -31,9 +27,10 @@ sub do_search {
     $self->config->{view_on_single_result} = 1;
 
     my $tmpf = File::Temp->new;
-    
-    my $file = $c->model( $self->model_name )->new_object( file => $tmpf->filename );
-    
+
+    my $file = $c->model( $self->model_name )
+        ->new_object( file => $tmpf->filename );
+
     if ( my $uri = $self->view_on_single_result( $c, [$file] ) ) {
         $c->response->redirect($uri);
         return;
@@ -43,5 +40,4 @@ sub do_search {
 
 }
 
-
 1;
